@@ -14,8 +14,10 @@
 # limitations under the License.
 
 import struct
+import socket
 from . import packet_base
 from . import packet_utils
+from . import icmp
 from . import udp
 from . import tcp
 from ryu.ofproto import inet
@@ -50,8 +52,8 @@ class ipv4(packet_base.PacketBase):
          src, dst) = struct.unpack_from(cls._PACK_STR, buf)
         header_length = version & 0xf
         version = version >> 4
-        offset = flags & ((1 << 15) - 1)
-        flags = flags >> 15
+        offset = flags & ((1 << 13) - 1)
+        flags = flags >> 13
         msg = cls(version, header_length, tos, total_length, identification,
                   flags, offset, ttl, proto, csum, src, dst)
 
@@ -61,17 +63,23 @@ class ipv4(packet_base.PacketBase):
         return msg, ipv4.get_packet_type(proto)
 
     def serialize(self, payload, prev):
-        hdr = bytearray().zfill(self.header_length * 4)
+        hdr = bytearray(self.header_length * 4)
         version = self.version << 4 | self.header_length
-        flags = self.flags << 15 | self.offset
+        flags = self.flags << 13 | self.offset
         if self.total_length == 0:
             self.total_length = self.header_length * 4 + len(payload)
         struct.pack_into(ipv4._PACK_STR, hdr, 0, version, self.tos,
                          self.total_length, self.identification, flags,
                          self.ttl, self.proto, 0, self.src, self.dst)
-        self.csum = packet_utils.checksum(hdr)
-        struct.pack_into('H', hdr, 10, self.csum)
+
+        if self.option:
+            assert (self.length - ipv4._MIN_LEN) >= len(self.option)
+            hdr[ipv4._MIN_LEN:ipv4._MIN_LEN + len(self.option)] = self.option
+
+        self.csum = socket.htons(packet_utils.checksum(hdr))
+        struct.pack_into('!H', hdr, 10, self.csum)
         return hdr
 
+ipv4.register_packet_type(icmp.icmp, inet.IPPROTO_ICMP)
 ipv4.register_packet_type(tcp.tcp, inet.IPPROTO_TCP)
 ipv4.register_packet_type(udp.udp, inet.IPPROTO_UDP)
