@@ -18,26 +18,18 @@
 import logging
 import itertools
 
-from ryu.ofproto import nx_match
+from ryu.ofproto import ofproto_v1_2
+from ryu.ofproto import ether
+from ryu.ofproto import inet
 from ryu.tests.integrated import tester
 
 LOG = logging.getLogger(__name__)
 
-IPPROTO_ICMP = 1
-IPPROTO_TCP = 6
-IPPROTO_UDP = 17
-IPPROTO_ROUTING = 43
-IPPROTO_FRAGMENT = 44
-IPPROTO_AH = 51
-IPPROTO_ICMPV6 = 58
-IPPROTO_NONE = 59
-IPPROTO_DSTOPTS = 60
-IPPROTO_SCTP = 132
-
 
 class RunTest(tester.TestFlowBase):
     """ Test case for add flows of Matches
     """
+    OFP_VERSIONS = [ofproto_v1_2.OFP_VERSION]
 
     def __init__(self, *args, **kwargs):
         super(RunTest, self).__init__(*args, **kwargs)
@@ -72,11 +64,11 @@ class RunTest(tester.TestFlowBase):
 
         if f_value == value and f_mask == mask:
             return True
-        elif value == None:
+        elif value is None:
             return "Field[%s] is setting." % (headers, )
         else:
             return "Value error. send: (%s/%s), val:(%s/%s)" \
-                        % (value, mask, f_value, f_mask)
+                % (value, mask, f_value, f_mask)
 
     def test_rule_set_dl_dst(self, dp):
         dl_dst = 'e2:7a:09:79:0b:0f'
@@ -187,7 +179,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_dl_type_ip(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         self._verify = [(dp.ofproto.OXM_OF_ETH_TYPE, ),
                         dl_type, None]
 
@@ -196,7 +188,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_dl_type_arp(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         self._verify = [(dp.ofproto.OXM_OF_ETH_TYPE, ),
                         dl_type, None]
 
@@ -205,7 +197,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_dl_type_vlan(self, dp):
-        dl_type = nx_match.ETH_TYPE_VLAN
+        dl_type = ether.ETH_TYPE_8021Q
         self._verify = [(dp.ofproto.OXM_OF_ETH_TYPE, ),
                         dl_type, None]
 
@@ -214,7 +206,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_dl_type_ipv6(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         self._verify = [(dp.ofproto.OXM_OF_ETH_TYPE, ),
                         dl_type, None]
 
@@ -223,7 +215,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_dl_type_lacp(self, dp):
-        dl_type = nx_match.ETH_TYPE_LACP
+        dl_type = ether.ETH_TYPE_SLOW
         self._verify = [(dp.ofproto.OXM_OF_ETH_TYPE, ),
                         dl_type, None]
 
@@ -233,7 +225,7 @@ class RunTest(tester.TestFlowBase):
 
     def test_rule_set_ip_dscp(self, dp):
         ip_dscp = 36
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         self._verify = [(dp.ofproto.OXM_OF_IP_DSCP, ),
                         ip_dscp, None]
 
@@ -297,7 +289,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_ecn(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         ip_ecn = 3
         self._verify = [(dp.ofproto.OXM_OF_IP_ECN, ),
                         ip_ecn, None]
@@ -308,8 +300,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_icmp(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_ICMP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_ICMP
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -319,8 +311,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_tcp(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_TCP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_TCP
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -330,8 +322,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_udp(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_UDP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_UDP
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -341,8 +333,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_ipv6_route(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ROUTING
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ROUTING
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -352,8 +344,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_ipv6_frag(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_FRAGMENT
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_FRAGMENT
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -363,8 +355,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_ipv6_icmp(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ICMPV6
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ICMPV6
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -374,8 +366,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_ipv6_none(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_NONE
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_NONE
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -385,8 +377,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ip_proto_ipv6_dstopts(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_DSTOPTS
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_DSTOPTS
         self._verify = [(dp.ofproto.OXM_OF_IP_PROTO, ),
                         ip_proto, None]
 
@@ -396,7 +388,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_src(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         src = '192.168.196.250'
         src_int = self.ipv4_to_int(src)
         self._verify = [(dp.ofproto.OXM_OF_IPV4_SRC,
@@ -409,7 +401,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_src_masked_32(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         src = '192.168.196.250'
         src_int = self.ipv4_to_int(src)
         mask = '255.255.255.255'
@@ -424,7 +416,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_src_masked_24(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         src = '192.168.196.250'
         src_int = self.ipv4_to_int(src)
         mask = '255.255.255.0'
@@ -439,7 +431,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_src_masked_0(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         src = '192.168.196.250'
         src_int = self.ipv4_to_int(src)
         mask = '0.0.0.0'
@@ -454,7 +446,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_dst(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         dst = '192.168.54.155'
         dst_int = self.ipv4_to_int(dst)
         self._verify = [(dp.ofproto.OXM_OF_IPV4_DST,
@@ -467,7 +459,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_dst_masked_32(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         dst = '192.168.54.155'
         dst_int = self.ipv4_to_int(dst)
         mask = '255.255.255.255'
@@ -482,7 +474,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_dst_masked_24(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         dst = '192.168.54.155'
         dst_int = self.ipv4_to_int(dst)
         mask = '255.255.255.0'
@@ -497,7 +489,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv4_dst_masked_0(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
+        dl_type = ether.ETH_TYPE_IP
         dst = '192.168.54.155'
         dst_int = self.ipv4_to_int(dst)
         mask = '0.0.0.0'
@@ -512,8 +504,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_tcp_src(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_TCP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_TCP
         tp_src = 1103
         self._verify = [(dp.ofproto.OXM_OF_TCP_SRC, ),
                         tp_src, None]
@@ -525,8 +517,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_tcp_dst(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_TCP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_TCP
         tp_dst = 236
         self._verify = [(dp.ofproto.OXM_OF_TCP_DST, ),
                         tp_dst, None]
@@ -538,8 +530,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_udp_src(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_UDP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_UDP
         tp_src = 56617
         self._verify = [(dp.ofproto.OXM_OF_UDP_SRC, ),
                         tp_src, None]
@@ -551,8 +543,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_udp_dst(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_UDP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_UDP
         tp_dst = 61278
         self._verify = [(dp.ofproto.OXM_OF_UDP_DST, ),
                         tp_dst, None]
@@ -564,8 +556,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_icmpv4_type(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_ICMP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_ICMP
         icmp_type = 8
         self._verify = [(dp.ofproto.OXM_OF_ICMPV4_TYPE, ),
                         icmp_type, None]
@@ -577,8 +569,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_icmpv4_code(self, dp):
-        dl_type = nx_match.ETH_TYPE_IP
-        ip_proto = IPPROTO_ICMP
+        dl_type = ether.ETH_TYPE_IP
+        ip_proto = inet.IPPROTO_ICMP
         icmp_type = 9
         icmp_code = 16
         self._verify = [(dp.ofproto.OXM_OF_ICMPV4_CODE, ),
@@ -592,7 +584,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_opcode(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_op = 1
         self._verify = [(dp.ofproto.OXM_OF_ARP_OP, ),
                         arp_op, None]
@@ -603,7 +595,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_spa(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_src = '192.168.222.57'
         nw_src_int = self.ipv4_to_int(nw_src)
         self._verify = [(dp.ofproto.OXM_OF_ARP_SPA,
@@ -616,7 +608,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_spa_masked_32(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_src = '192.168.222.57'
         nw_src_int = self.ipv4_to_int(nw_src)
         mask = '255.255.255.255'
@@ -631,7 +623,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_spa_masked_24(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_src = '192.168.222.57'
         nw_src_int = self.ipv4_to_int(nw_src)
         mask = '255.255.255.0'
@@ -646,7 +638,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_spa_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_src = '192.168.222.57'
         nw_src_int = self.ipv4_to_int(nw_src)
         mask = '0.0.0.0'
@@ -661,7 +653,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tpa(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_dst = '192.168.198.233'
         nw_dst_int = self.ipv4_to_int(nw_dst)
         self._verify = [(dp.ofproto.OXM_OF_ARP_TPA,
@@ -674,7 +666,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tpa_masked_32(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_dst = '192.168.198.233'
         nw_dst_int = self.ipv4_to_int(nw_dst)
         mask = '255.255.255.255'
@@ -689,7 +681,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tpa_masked_24(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_dst = '192.168.198.233'
         nw_dst_int = self.ipv4_to_int(nw_dst)
         mask = '255.255.255.0'
@@ -704,7 +696,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tpa_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         nw_dst = '192.168.198.233'
         nw_dst_int = self.ipv4_to_int(nw_dst)
         mask = '0.0.0.0'
@@ -719,7 +711,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_sha(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_sha = '3e:ec:13:9b:f3:0b'
         arp_sha_bin = self.haddr_to_bin(arp_sha)
         self._verify = [(dp.ofproto.OXM_OF_ARP_SHA,
@@ -732,7 +724,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_sha_masked_ff(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_sha = '3e:ec:13:9b:f3:0b'
         arp_sha_bin = self.haddr_to_bin(arp_sha)
         mask = 'ff:ff:ff:ff:ff:ff'
@@ -747,7 +739,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_sha_masked_f0(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_sha = '3e:ec:13:9b:f3:0b'
         arp_sha_bin = self.haddr_to_bin(arp_sha)
         mask = 'ff:ff:ff:ff:ff:00'
@@ -762,7 +754,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_sha_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_sha = '3e:ec:13:9b:f3:0b'
         arp_sha_bin = self.haddr_to_bin(arp_sha)
         mask = '00:00:00:00:00:00'
@@ -777,7 +769,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tha(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_tha = '83:6c:21:52:49:68'
         arp_tha_bin = self.haddr_to_bin(arp_tha)
         self._verify = [(dp.ofproto.OXM_OF_ARP_THA,
@@ -790,7 +782,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tha_masked_ff(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_tha = '83:6c:21:52:49:68'
         arp_tha_bin = self.haddr_to_bin(arp_tha)
         mask = 'ff:ff:ff:ff:ff:ff'
@@ -805,7 +797,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tha_masked_f0(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_tha = '83:6c:21:52:49:68'
         arp_tha_bin = self.haddr_to_bin(arp_tha)
         mask = 'ff:ff:ff:ff:ff:00'
@@ -820,7 +812,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_arp_tha_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_ARP
+        dl_type = ether.ETH_TYPE_ARP
         arp_tha = '83:6c:21:52:49:68'
         arp_tha_bin = self.haddr_to_bin(arp_tha)
         mask = '00:00:00:00:00:00'
@@ -835,7 +827,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_src(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_src = '2001:db8:bd05:1d2:288a:1fc0:1:10ee'
         ipv6_src_int = self.ipv6_to_int(ipv6_src)
         self._verify = [(dp.ofproto.OXM_OF_IPV6_SRC,
@@ -848,7 +840,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_src_masked_ff(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_src = '2001:db8:bd05:1d2:288a:1fc0:1:10ee'
         ipv6_src_int = self.ipv6_to_int(ipv6_src)
         mask = 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff'
@@ -863,13 +855,13 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_src_masked_f0(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_src = '2001:db8:bd05:1d2:288a:1fc0:1:10ee'
         ipv6_src_int = self.ipv6_to_int(ipv6_src)
         mask = 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:0'
         mask_int = self.ipv6_to_int(mask)
         ipv6_src_masked = [x & y for (x, y) in
-                              itertools.izip(ipv6_src_int, mask_int)]
+                           itertools.izip(ipv6_src_int, mask_int)]
         self._verify = [(dp.ofproto.OXM_OF_IPV6_SRC,
                          dp.ofproto.OXM_OF_IPV6_SRC_W, ),
                         ipv6_src_masked, mask_int]
@@ -880,7 +872,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_src_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_src = '2001:db8:bd05:1d2:288a:1fc0:1:10ee'
         ipv6_src_int = self.ipv6_to_int(ipv6_src)
         mask = '0:0:0:0:0:0:0:0'
@@ -895,7 +887,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_dst(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_dst = 'e9e8:9ea5:7d67:82cc:ca54:1fc0:2d24:f038'
         ipv6_dst_int = self.ipv6_to_int(ipv6_dst)
         self._verify = [(dp.ofproto.OXM_OF_IPV6_DST,
@@ -908,7 +900,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_dst_masked_ff(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_dst = 'e9e8:9ea5:7d67:82cc:ca54:1fc0:2d24:f038'
         ipv6_dst_int = self.ipv6_to_int(ipv6_dst)
         mask = 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff'
@@ -923,13 +915,13 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_dst_masked_f0(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_dst = 'e9e8:9ea5:7d67:82cc:ca54:1fc0:2d24:f038'
         ipv6_dst_int = self.ipv6_to_int(ipv6_dst)
         mask = 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:0'
         mask_int = self.ipv6_to_int(mask)
         ipv6_dst_masked = [x & y for (x, y) in
-                              itertools.izip(ipv6_dst_int, mask_int)]
+                           itertools.izip(ipv6_dst_int, mask_int)]
         self._verify = [(dp.ofproto.OXM_OF_IPV6_DST,
                          dp.ofproto.OXM_OF_IPV6_DST_W, ),
                         ipv6_dst_masked, mask_int]
@@ -940,7 +932,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_dst_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_dst = 'e9e8:9ea5:7d67:82cc:ca54:1fc0:2d24:f038'
         ipv6_dst_int = self.ipv6_to_int(ipv6_dst)
         mask = '0:0:0:0:0:0:0:0'
@@ -955,7 +947,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_flabel(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_label = 0xc5384
         self._verify = [(dp.ofproto.OXM_OF_IPV6_FLABEL,
                          dp.ofproto.OXM_OF_IPV6_FLABEL_W, ),
@@ -967,7 +959,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_flabel_masked_ff(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_label = 0xc5384
         mask = 0xfffff
         self._verify = [(dp.ofproto.OXM_OF_IPV6_FLABEL,
@@ -980,7 +972,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_flabel_masked_f0(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_label = 0xc5384
         mask = 0xffff0
         self._verify = [(dp.ofproto.OXM_OF_IPV6_FLABEL,
@@ -993,7 +985,7 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_flabel_masked_00(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
+        dl_type = ether.ETH_TYPE_IPV6
         ipv6_label = 0xc5384
         mask = 0x0
         self._verify = [(dp.ofproto.OXM_OF_IPV6_FLABEL,
@@ -1006,8 +998,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_icmpv6_type(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ICMPV6
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ICMPV6
         icmp_type = 129
         self._verify = [(dp.ofproto.OXM_OF_ICMPV6_TYPE, ),
                         icmp_type, None]
@@ -1019,8 +1011,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_icmpv6_code(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ICMPV6
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ICMPV6
         icmp_type = 138
         icmp_code = 1
         self._verify = [(dp.ofproto.OXM_OF_ICMPV6_CODE, ),
@@ -1034,8 +1026,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_nd_target(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ICMPV6
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ICMPV6
         icmp_type = 135
         target = "5420:db3f:921b:3e33:2791:98f:dd7f:2e19"
         target_int = self.ipv6_to_int(target)
@@ -1050,8 +1042,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_nd_sll(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ICMPV6
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ICMPV6
         icmp_type = 135
         nd_sll = "93:6d:d0:d4:e8:36"
         nd_sll_bin = self.haddr_to_bin(nd_sll)
@@ -1066,8 +1058,8 @@ class RunTest(tester.TestFlowBase):
         self.add_matches(dp, match)
 
     def test_rule_set_ipv6_nd_tll(self, dp):
-        dl_type = nx_match.ETH_TYPE_IPV6
-        ip_proto = IPPROTO_ICMPV6
+        dl_type = ether.ETH_TYPE_IPV6
+        ip_proto = inet.IPPROTO_ICMPV6
         icmp_type = 136
         nd_tll = "18:f6:66:b6:f1:b3"
         nd_tll_bin = self.haddr_to_bin(nd_tll)
